from flask import Flask, jsonify, request
from uuid import uuid4
import uuid
import requests


class Theme:
    def __init__(self, number, name, uuid_: uuid):
        if uuid_ is None:
            self.uuid_: uuid = uuid4()
        else:
            self.uuid_: uuid = uuid.UUID(uuid_)
        self.number: str = number
        self.name: str = name

    def to_dict(self):
        return {
            "uuid": self.uuid_,
            "number": self.number,
            "name": self.name
        }

    def to_dict_with_books(self, books: list):
        return {
            "uuid": self.uuid_,
            "number": self.number,
            "name": self.name,
            "books": books
        }


app = Flask(__name__)

themes: list[Theme] = [
    Theme(name='Flowers', number='1', uuid_='997aa4c5-ebb2-4794-ba81-e742f9f1fa30'),
    Theme(name='Art', number='2', uuid_='694827e4-0f93-45a5-8f75-bad7ef2d21fe')
]

books_url = 'http://book_service:20002/'


def list_jsonify():
    return jsonify([theme.to_dict() for theme in themes])


# получение списка всех тем
@app.route('/', methods=['GET'])
def get_all():
    return list_jsonify(), 200


# получение темы по идентификатору
@app.route('/<uuid:uuid_>', methods=['GET'])
def get_one(uuid_):
    for theme in themes:
        if theme.uuid_ == uuid_:
            return theme.to_dict(), 200

    return 'Тема с таким uuid не была найдена', 404


# получение темы со списком  книг по ней
@app.route('/with-books/<uuid:uuid_>', methods=['GET'])
def get_one_with_books(uuid_):
    for theme in themes:
        if theme.uuid_ == uuid_:
            response = requests.get(books_url + f'by-theme/{uuid_}')
            print(response.json())
            return theme.to_dict_with_books(response.json()), 200

    return 'Тема с таким uuid не была найдена', 404


# создание темы
@app.route('/', methods=['POST'])
def create():
    data = request.json
    number = data.get('number', None)
    name = data.get('name', None)
    if name is None or number is None:
        return 'Недостаточно полей для создания новой темы', 404

    new_theme = Theme(number, name, None)
    themes.append(new_theme)
    return get_one(new_theme.uuid_)


# изменение темы по идентификатору
@app.route('/<uuid:uuid_>', methods=['PUT'])
def update_by_id(uuid_):
    data = request.json
    new_number = data.get('number', None)
    new_name = data.get('name', None)

    for theme in themes:
        if theme.uuid_ == uuid_:
            if new_number is not None:
                theme.number = new_number
            if new_name is not None:
                theme.name = new_name
            return get_one(theme.uuid_)

    return 'Тема с таким uuid не была найдена', 404


# удаление темы по идентификатору
@app.route('/<uuid:uuid_>', methods=['DELETE'])
def delete(uuid_):
    for theme in themes:
        if theme.uuid_ == uuid_:
            themes.remove(theme)
            return 'Тема успешно удалена', 200

    return 'Тема с таким uuid не была найдена', 404


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=20001, debug=True)